from .move_setup_py_to_declarative import MoveSetupPyToDeclarativeTest
from .move_setup_requires import MoveSetupRequiresTest

__all__ = [
    "MoveSetupRequiresTest",
    "MoveSetupPyToDeclarativeTest",
]
